/* Copyright (c) 2006-2011, Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package dk.vitality.lib.connectionCache;

import org.jetbrains.annotations.NotNull;

public abstract class TransientConnectionRunnable<INFO, CONN>
{
	public abstract void run() throws Exception;

	@NotNull
	public synchronized CONN obtainConnection() throws Exception
	{
		if (cache == null)
			throw new IllegalStateException("Out of scope for connections");
		if (connInfo == null)
			connInfo = cache.get();
		return connInfo.connInfo.getConnection();
	}

	public final boolean hasConnection()
	{
		return connInfo != null;
	}

	public final synchronized void releaseConnection()
	{
		if (connInfo == null)
			return;
		if (cache == null)
			throw new IllegalStateException("Out of scope for connections");
		cache.catchConnection(connInfo);
		connInfo = null;
	}

	public final synchronized void discardBusyConnection()
	{
		if (connInfo == null)
			return;
		if (cache == null)
			throw new IllegalStateException("Out of scope for connections");
		cache.discardBusyConnection(connInfo, "Explicit discard");
		connInfo = null;
	}

	/* package */
	final void notifyEnterScope(@NotNull ConnectionCache<INFO, CONN> cache)
	{
		notifyLeaveScope();
		this.cache = cache;
	}

	/* package */
	final void notifyLeaveScope()
	{
		discardBusyConnection();
		cache = null;
	}

	private ConnectionCache<INFO, CONN> cache;
	private ConnectionCache<INFO, CONN>.PrivateConnectionInfo connInfo;
}
