/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.runtime.lib;

public enum Opcode
{
	PRINT_STRINGTABLE,
	PRINT_STRING_VAR,
	PRINT_HTML_ENCODED_STRING_VAR,
	PRINT_ATTR_ENCODED_STRING_VAR,
	PRINT_INT_VAR,
	CALLBACK,
	GOTO, CONDITIONAL_GOTO,
	NEGATE,
	RETURN,
	TEST_IS_STRING_NULL,
	// ADD TO the end to keep current compiled code valid.
	;

	public static final int OPCODE_BITS = 4;
	public static final int OPCODE_MASK = (1 << OPCODE_BITS) - 1;

	public int encoded()
	{
		return ordinal();
	}

	public int encoded(int param)
	{
		return ordinal() | (param << OPCODE_BITS);
	}

	public static Opcode decodeOpcode(int encoded)
	{
		return opcodes[encoded & OPCODE_MASK];
	}

	public static int decodeParam(int encoded)
	{
		return encoded >>> OPCODE_BITS;
	}

	public static String toString(int encoded)
	{
		Opcode opcode = Opcode.decodeOpcode(encoded);
		switch (opcode) {
			case PRINT_STRINGTABLE:
			case CALLBACK:
			case GOTO:
			case CONDITIONAL_GOTO:
				return opcode.name().toLowerCase() + "(" + decodeParam(encoded) + ")";
			default:
				return opcode.name().toLowerCase();
		}
	}

	private static Opcode[] opcodes = values();
	static {
		if ((opcodes.length & ~OPCODE_MASK) != 0)
			throw new RuntimeException("Too few OPCODE_BITS for the current number of opcodes.");
	}
}
