/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.application.scope;

import java.lang.annotation.Annotation;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.jabhts.annotation.attribute.collection.ACollection;
import org.jabhts.annotation.attribute.collection.ASrcWidthHeight;
import org.jabhts.annotation.attribute.collection.ASrcWidthHeightAlt;
import org.jabhts.annotation.attribute.content.AAction;
import org.jabhts.annotation.attribute.content.AAlign;
import org.jabhts.annotation.attribute.content.AAlt;
import org.jabhts.annotation.attribute.content.AAutocomplete;
import org.jabhts.annotation.attribute.content.AAutofocus;
import org.jabhts.annotation.attribute.content.ACharset;
import org.jabhts.annotation.attribute.content.AChecked;
import org.jabhts.annotation.attribute.content.AClass;
import org.jabhts.annotation.attribute.content.AClassPost;
import org.jabhts.annotation.attribute.content.AClear;
import org.jabhts.annotation.attribute.content.AColSpan;
import org.jabhts.annotation.attribute.content.ACoords;
import org.jabhts.annotation.attribute.content.AEnabled;
import org.jabhts.annotation.attribute.content.AEncType;
import org.jabhts.annotation.attribute.content.AFor;
import org.jabhts.annotation.attribute.content.AHeight;
import org.jabhts.annotation.attribute.content.AHref;
import org.jabhts.annotation.attribute.content.AId;
import org.jabhts.annotation.attribute.content.AIsMap;
import org.jabhts.annotation.attribute.content.ALang;
import org.jabhts.annotation.attribute.content.AManifest;
import org.jabhts.annotation.attribute.content.AMultiple;
import org.jabhts.annotation.attribute.content.AName;
import org.jabhts.annotation.attribute.content.AOnBlur;
import org.jabhts.annotation.attribute.content.AOnChange;
import org.jabhts.annotation.attribute.content.AOnClick;
import org.jabhts.annotation.attribute.content.AOnDblClick;
import org.jabhts.annotation.attribute.content.AOnError;
import org.jabhts.annotation.attribute.content.AOnFocus;
import org.jabhts.annotation.attribute.content.AOnKeyDown;
import org.jabhts.annotation.attribute.content.AOnKeyPress;
import org.jabhts.annotation.attribute.content.AOnKeyUp;
import org.jabhts.annotation.attribute.content.AOnLoad;
import org.jabhts.annotation.attribute.content.AOnMouseDown;
import org.jabhts.annotation.attribute.content.AOnMouseEnter;
import org.jabhts.annotation.attribute.content.AOnMouseLeave;
import org.jabhts.annotation.attribute.content.AOnMouseMove;
import org.jabhts.annotation.attribute.content.AOnMouseOut;
import org.jabhts.annotation.attribute.content.AOnMouseOver;
import org.jabhts.annotation.attribute.content.AOnMouseUp;
import org.jabhts.annotation.attribute.content.AOnReset;
import org.jabhts.annotation.attribute.content.AOnSelect;
import org.jabhts.annotation.attribute.content.AOnSubmit;
import org.jabhts.annotation.attribute.content.AOnUnload;
import org.jabhts.annotation.attribute.content.APlaceholder;
import org.jabhts.annotation.attribute.content.AReadOnly;
import org.jabhts.annotation.attribute.content.ARowSpan;
import org.jabhts.annotation.attribute.content.ASelected;
import org.jabhts.annotation.attribute.content.ASrc;
import org.jabhts.annotation.attribute.content.AStyle;
import org.jabhts.annotation.attribute.content.AStylePost;
import org.jabhts.annotation.attribute.content.ATabIndex;
import org.jabhts.annotation.attribute.content.ATarget;
import org.jabhts.annotation.attribute.content.ATitle;
import org.jabhts.annotation.attribute.content.AType;
import org.jabhts.annotation.attribute.content.AUseMap;
import org.jabhts.annotation.attribute.content.AValign;
import org.jabhts.annotation.attribute.content.AValue;
import org.jabhts.annotation.attribute.content.AWidth;
import org.jabhts.annotation.attribute.sub.SCollection;
import org.jabhts.annotation.attribute.sub.SDisplay;
import org.jabhts.annotation.attribute.sub.SDisplayBlock;
import org.jabhts.annotation.attribute.sub.SDisplayInline;
import org.jabhts.annotation.attribute.sub.SHasClass;
import org.jabhts.annotation.attribute.sub.SValue;
import org.jabhts.annotation.element.EHtml;
import org.jabhts.annotation.element.EHtmlPost;
import org.jabhts.annotation.element.EHtmlPre;
import org.jabhts.annotation.element.EReplace;
import org.jabhts.annotation.element.EText;
import org.jabhts.annotation.element.ETextPost;
import org.jabhts.annotation.element.ETextPre;
import org.jabhts.annotation.flowControl.EContentIf;
import org.jabhts.annotation.flowControl.EContentIfNot;
import org.jabhts.annotation.flowControl.EIf;
import org.jabhts.annotation.flowControl.EIfNot;
import org.jabhts.annotation.flowControl.EIterate;
import org.jabhts.annotation.flowControl.EScopeChilds;
import org.jabhts.annotation.flowControl.EScopeTag;
import org.jabhts.annotation.flowControl.EScopingWhile;
import org.jabhts.annotation.flowControl.ETagIf;
import org.jabhts.annotation.flowControl.ETagIfNot;
import org.jabhts.annotation.flowControl.EWhile;
import org.jabhts.annotation.flowControl.EZap;
import org.jabhts.translator.compiler.application.attribute.collection.ACollectionApplication;
import org.jabhts.translator.compiler.application.attribute.collection.ASrcWidthHeightApplication;
import org.jabhts.translator.compiler.application.attribute.content.BooleanAttributeApplication;
import org.jabhts.translator.compiler.application.attribute.content.IntegerAttributeApplication;
import org.jabhts.translator.compiler.application.attribute.content.StringAttributeApplication;
import org.jabhts.translator.compiler.application.attribute.content.StringPostAttributeApplication;
import org.jabhts.translator.compiler.application.attribute.sub.SCollectionApplication;
import org.jabhts.translator.compiler.application.attribute.sub.SDisplayApplication;
import org.jabhts.translator.compiler.application.attribute.sub.SDisplayBlockApplication;
import org.jabhts.translator.compiler.application.attribute.sub.SDisplayInlineApplication;
import org.jabhts.translator.compiler.application.attribute.sub.SHasClassApplication;
import org.jabhts.translator.compiler.application.attribute.sub.SValueApplication;
import org.jabhts.translator.compiler.application.element.scoped.EIterateApplication;
import org.jabhts.translator.compiler.application.element.scoped.EScopeChildsApplication;
import org.jabhts.translator.compiler.application.element.scoped.EScopeTagApplication;
import org.jabhts.translator.compiler.application.element.scoped.EScopingWhileApplication;
import org.jabhts.translator.compiler.application.element.simple.EContentApplication;
import org.jabhts.translator.compiler.application.element.simple.EContentIfApplication;
import org.jabhts.translator.compiler.application.element.simple.EContentIfNotApplication;
import org.jabhts.translator.compiler.application.element.simple.EContentPrePostApplication;
import org.jabhts.translator.compiler.application.element.simple.EIfApplication;
import org.jabhts.translator.compiler.application.element.simple.EIfNotApplication;
import org.jabhts.translator.compiler.application.element.simple.EReplaceApplication;
import org.jabhts.translator.compiler.application.element.simple.ETagIfApplication;
import org.jabhts.translator.compiler.application.element.simple.ETagIfNotApplication;
import org.jabhts.translator.compiler.application.element.simple.EWhileApplication;
import org.jabhts.translator.compiler.application.element.simple.EZapApplication;
import static org.jabhts.translator.compiler.application.baseclass.AbstractStringApplication.EncodingType;

class AnnotationApplicationMap
{
	public static ApplicationFactory get(Annotation annotation)
	{
		return map.get(annotation.annotationType().getName());
	}

	private static final Map<String, ApplicationFactory> map = new HashMap<>();


	static {
		addStringAttr("autocomplete", AAutocomplete.class);
		addStringAttr("action", AAction.class);
		addStringAttr("align", AAlign.class);
		addStringAttr("alt", AAlt.class);
		addStringAttr("charset", ACharset.class);
		addStringAttr("class", AClass.class);
		addStringPostAttr("class", AClassPost.class, " ");
		addStringAttr("clear", AClear.class);
		addStringAttr("coords", ACoords.class);
		addStringAttr("enctype", AEncType.class);
		addStringAttr("for", AFor.class);
		addStringAttr("height", AHeight.class);
		addStringAttr("href", AHref.class);
		addStringAttr("id", AId.class);
		addStringAttr("lang", ALang.class);
		addStringAttr("manifest", AManifest.class);
		addStringAttr("name", AName.class);
		addStringAttr("onblur", AOnBlur.class);
		addStringAttr("onclick", AOnClick.class);
		addStringAttr("onchange", AOnChange.class);
		addStringAttr("ondblclick", AOnDblClick.class);
		addStringAttr("onerror", AOnError.class);
		addStringAttr("onfocus", AOnFocus.class);
		addStringAttr("onkeydown", AOnKeyDown.class);
		addStringAttr("onkeypress", AOnKeyPress.class);
		addStringAttr("onkeyup", AOnKeyUp.class);
		addStringAttr("onload", AOnLoad.class);
		addStringAttr("onmousedown", AOnMouseDown.class);
		addStringAttr("onmouseenter", AOnMouseEnter.class);
		addStringAttr("onmouseleave", AOnMouseLeave.class);
		addStringAttr("onmousemove", AOnMouseMove.class);
		addStringAttr("onmouseout", AOnMouseOut.class);
		addStringAttr("onmouseover", AOnMouseOver.class);
		addStringAttr("onmouseup", AOnMouseUp.class);
		addStringAttr("onreset", AOnReset.class);
		addStringAttr("onselect", AOnSelect.class);
		addStringAttr("onsubmit", AOnSubmit.class);
		addStringAttr("onunload", AOnUnload.class);
		addStringAttr("placeholder", APlaceholder.class);
		addStringAttr("src", ASrc.class);
		addStringAttr("style", AStyle.class);
		addStringPostAttr("style", AStylePost.class, "; ");
		addStringAttr("target", ATarget.class);
		addStringAttr("title", ATitle.class);
		addStringAttr("type", AType.class);
		addStringAttr("usemap", AUseMap.class);
		addStringAttr("valign", AValign.class);
		addStringAttr("value", AValue.class);
		addStringAttr("width", AWidth.class);
		//------
		addBooleanAttr("autofocus", AAutofocus.class);
		addBooleanAttr("checked", AChecked.class);
		addBooleanAttr("disabled", AEnabled.class);
		addBooleanAttr("multiple", AMultiple.class);
		addBooleanAttr("ismap", AIsMap.class);
		addBooleanAttr("readonly", AReadOnly.class);
		addBooleanAttr("selected", ASelected.class);
		//------
		addIntegerAttr("colspan", AColSpan.class);
		addIntegerAttr("rowspan", ARowSpan.class);
		addIntegerAttr("tabindex", ATabIndex.class);

		//-----------------------------------------
		add(ASrcWidthHeight.class, (scope, target) -> new ASrcWidthHeightApplication(target, "src", "width", "height"));
		add(ASrcWidthHeightAlt.class,
				(scope, target) -> new ASrcWidthHeightApplication(target, "src", "width", "height", "alt"));
		add(ACollection.class, (scope, target) -> new ACollectionApplication(target));
		//-----------------------------------------
		add(SHasClass.class, (scope, target) -> new SHasClassApplication(target));
		//-----------------------------------------
		add(SDisplay.class, (scope, target) -> new SDisplayApplication(target));
		add(SDisplayBlock.class, (scope, target) -> new SDisplayBlockApplication(target));
		add(SDisplayInline.class, (scope, target) -> new SDisplayInlineApplication(target));
		add(SValue.class, (scope, target) -> new SValueApplication(target));
		add(SCollection.class, (scope, target) -> new SCollectionApplication(target));
		//-----------------------------------------

		add(EText.class, (scope, target) -> new EContentApplication(target, EncodingType.Html));
		add(ETextPre.class, (scope, target) -> new EContentPrePostApplication(target, EncodingType.Html, true));
		add(ETextPost.class, (scope, target) -> new EContentPrePostApplication(target, EncodingType.Html, false));
		add(EHtml.class, (scope, target) -> new EContentApplication(target, EncodingType.Raw));
		add(EHtmlPre.class, (scope, target) -> new EContentPrePostApplication(target, EncodingType.Raw, true));
		add(EHtmlPost.class, (scope, target) -> new EContentPrePostApplication(target, EncodingType.Raw, false));
		add(EReplace.class, (scope, target) -> new EReplaceApplication(target));
		add(EZap.class, (scope, target) -> new EZapApplication(target));
		add(EIf.class, (scope, target) -> new EIfApplication(target));
		add(EIfNot.class, (scope, target) -> new EIfNotApplication(target));
		add(EContentIf.class, (scope, target) -> new EContentIfApplication(target));
		add(EContentIfNot.class, (scope, target) -> new EContentIfNotApplication(target));
		add(ETagIf.class, (scope, target) -> new ETagIfApplication(target));
		add(ETagIfNot.class, (scope, target) -> new ETagIfNotApplication(target));
		add(EWhile.class, (scope, target) -> new EWhileApplication(target));
		add(EScopeChilds.class, (scope, target) -> new EScopeChildsApplication(target));
		add(EScopeTag.class, (scope, target) -> new EScopeTagApplication(target));
		add(EScopingWhile.class, (scope, target) -> new EScopingWhileApplication(target));
		add(EIterate.class, (scope, target) -> new EIterateApplication(target));
		if (1 == 2) {
			System.err.println("Num: " + map.size());
			List<String> list = new ArrayList<>(map.keySet());
			list.sort(null);
			for (String s : list)
				System.err.println(s.substring(1 + s.lastIndexOf('.')));
		}
	}

	private static void add(Class<?> cls, ApplicationFactory factory)
	{
		map.put(cls.getName(), factory);
	}

	private static void addStringAttr(final String attrName, Class<?> cls)
	{
		add(cls, (scope, target) -> new StringAttributeApplication(target, attrName));
	}

	private static void addStringPostAttr(final String attrName, Class<?> cls, final String postContentSeparator)
	{
		add(cls, (scope, target) -> new StringPostAttributeApplication(target, postContentSeparator, attrName));
	}

	private static void addBooleanAttr(final String attrName, Class<?> cls)
	{
		add(cls, (scope, target) -> {
			boolean inverse = attrName.equals("disabled");
			return new BooleanAttributeApplication(target, attrName, inverse);
		});
	}

	private static void addIntegerAttr(final String attrName, Class<?> cls)
	{
		add(cls, (scope, target) -> new IntegerAttributeApplication(target, attrName));

	}
}
