/* Copyright (c) 2006-2011, Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package dk.vitality.lib.foundation.io;


import java.io.File;
import org.jetbrains.annotations.NotNull;

/**
 * For all suffix functions, suffix includes the dot
 */

public class FilenameUtils
{
	// This functions appends a suffix to a filename. It cuts off some of the
	// filename, if the resulting string is too long for MacOS.
	// In the future, we will be a no-op if not running under MacOS.
	// suffix should include dot.
	public static String addSuffixToFilename(@NotNull String filename, @NotNull String suffix)
	{
		if (filename.length() + suffix.length() <= 31)
			return filename + suffix;
		String s = filename.substring(0, 31 - suffix.length());
		return s + suffix;
	}

	/**
	 * Filename without the suffix
	 */
	public static String basename(@NotNull File f)
	{
		return basename(f.getName());
	}

	/**
	 * Filename without the suffix
	 */
	public static String basename(@NotNull String filename)
	{
		int i = filename.lastIndexOf('.');
		if (i >= Math.max(0, filename.length() - 7))
			filename = filename.substring(0, i);
		return filename;
	}

	/**
	 * @return The suffix including the dot.
	 */
	public static String suffix(@NotNull File f)
	{
		return suffix(f.getName());
	}

	/**
	 * @return The suffix including the dot.
	 */
	public static String suffix(@NotNull String filename)
	{
		int i = filename.lastIndexOf('.');
		return i >= 0 ? filename.substring(i) : "";
	}

	/**
	 * Removes the exiting suffix and adds the specified suffix.
	 *
	 * @param suffix Must include the dot.
	 */
	public static File changeSuffixTo(@NotNull File f, @NotNull String suffix)
	{
		return new File(f.getParentFile(), changeSuffixTo(f.getName(), suffix));
	}

	public static String changeSuffixTo(@NotNull String filename, @NotNull String suffix)
	{
		filename = basename(filename);
		if (suffix.isEmpty())
			return filename;
		return filename + suffix;
	}
}
