/* Copyright (c) 2006-2011, Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package dk.vitality.lib.foundation.misc;

import java.security.MessageDigest;
import org.jetbrains.annotations.NotNull;

public class HexUtils
{
	private static final char[] lcHexChars = {
			'0', '1', '2', '3', '4', '5', '6', '7',
			'8', '9', 'a', 'b', 'c', 'd', 'e', 'f',};

	@NotNull
	public static String toLcHex(@NotNull  MessageDigest digest)
	{
		return toLcHex(digest.digest());
	}

	@NotNull
	public static String toLcHex(@NotNull byte[] bytes)
	{
		return toLcHex(bytes, 0, bytes.length);
	}

	@NotNull
	public static String toLcHex(@NotNull byte[] bytes, int offset, int len)
	{
		if (len == 0)
			return "";
		char[] chars = new char[len * 2];
		int dst = 0;
		while (len > 0) {
			len--;
			byte b = bytes[offset++];
			chars[dst++] = lcHexChars[(b >>> 4) & 15];
			chars[dst++] = lcHexChars[b & 15];
		}
		return new String(chars);
	}

	@NotNull
	public static String toLcHexSpaced(@NotNull byte[] bytes)
	{
		return toLcHexSpaced(bytes, 0, bytes.length);
	}

	@NotNull
	public static String toLcHexSpaced(@NotNull byte[] bytes, int offset, int len)
	{
		if (len == 0)
			return "";
		char[] chars = new char[len * 3 - 1];
		int dst = 0;
		while (true) {
			len--;
			byte b = bytes[offset++];
			chars[dst++] = lcHexChars[(b >>> 4) & 15];
			chars[dst++] = lcHexChars[b & 15];
			if (len == 0)
				break;
			chars[dst++] = ' ';
		}
		return new String(chars);
	}
}
