/* Copyright (c) 2006-2011, Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package dk.vitality.lib.foundation.xml.xpath;

import java.util.ArrayList;
import java.util.List;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;


public abstract class PathBase
{
	int wantedIndex;
	ExpressionBase test;

	public abstract void execute(PathRuntime r, Element currentElement) throws SAXException;

	public void debugDump(String prefix) throws SAXException
	{
		List<String> list = new ArrayList<>();
		String cname = getClass().getName();
		cname = cname.substring(cname.lastIndexOf('.') + 1);
		list.add(prefix + cname);
		getAdditionalDebugInfo(list);
		StringBuilder sb = new StringBuilder(20);
		for (int i = 0; i < list.size(); i++) {
			if (i > 0)
				sb.append(", ");
			sb.append(list.get(i));
		}
		System.err.println(sb.toString());
		if (test != null)
			test.debugDump(6);
	}

	protected void getAdditionalDebugInfo(List<String> list) throws SAXException
	{
		if (wantedIndex > 0)
			list.add("index = " + wantedIndex);
	}
}

// END OF FILE
