#!/bin/sh

set -e

if [ -s make-jre-bundle.sh ]
then
	cd ..
fi

export JAVA_HOME=`/usr/libexec/java_home`
echo "JAVA_HOME: $JAVA_HOME"

PATH=$JAVA_HOME/bin:$PATH

GBTMP=../tmp
JAVATMP=../tmp/JVM

set -v
rm -rf $JAVATMP/*

mkdir -p $JAVATMP/classes
javac -d $JAVATMP/classes \
	./JVM/src/dk/Test.java \
	./JVM/src/SslUtil.java

mkdir -p $JAVATMP/jars
javapackager -createjar -appclass dk.Test -outdir $JAVATMP/jars -outfile tester -srcdir $JAVATMP/classes

javapackager -deploy -appclass dk.Test -native image \
	-Bmac.signing-key-user-name=NonexistentUsernameSoJREDoesntGetSigned \
	-outdir $JAVATMP -outfile tester -srcdir $JAVATMP/jars -srcfiles tester.jar

JRE=`echo $JAVATMP/bundles/Test.app/Contents/PlugIns/Java.runtime`
if [ ! -d $JRE ]
then
	echo "Missing JRE bundle: $JRE"
	exit 1
fi

rm -f ./JVM/jre-bundle
ln -s ../$JRE ./JVM/jre-bundle

rm -f ./JVM/jdk-include
ln -s $JAVA_HOME/include ./JVM/jdk-include

# add cert for StartSSL, used by glimmerblocker.org
# All browsers has the cert, it's just oracle that won't add it to java
CACERTS=JVM/jre-bundle/Contents//Home/jre/lib/security/cacerts
chmod +w $CACERTS
java -cp $JAVATMP/classes SslUtil $CACERTS
chmod -w $CACERTS

# rename some files so they always end with *.txt as codesign otherwise
# thinks they are executables.
for f in `find $JRE/ -name COPYRIGHT -o -name LICENSE`
do
	echo "Renames to $f.txt"
	mv "$f" "$f.txt"
done

# codesign doesn't like symlinks in executable folder,
# but not having such a folder is ok.

# Xcode 6.3 no longer likes having the JVM as a signed PlugIn.
#echo "codesigns JVM. This takes time."
#./bin/codesign.pl JVM/jre-bundle/
