/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import <Cocoa/Cocoa.h>
#import "VerticalViewFlow.h"

@class AlertHelper;
@class ServerCommunication;
@class Prefs;
@class FilterTabController;
@class FilterListView;

@interface Filter : VerticalViewFlowRow<NSTextFieldDelegate> {
@private
	int _filterId;
	NSString* _name;
	NSString* _description;
	NSString* _comments;
	NSString* _url;
	NSString* _updateError;
}

@property (readonly) BOOL safetySimpleRulesOnly;
@property (readonly) Prefs* prefs;
@property (readonly) NSXMLElement* filterE;
@property (readonly) NSXMLElement* filterContainerE; // only subscriptions
@property (readonly) NSXMLElement* filterDataE;
@property (readonly) BOOL enabled;
@property (readonly) BOOL isSubscription;


- (instancetype)initWithFilterElement:(NSXMLElement*)filterE
							 withComm:(ServerCommunication*)comm
					withTabController:(FilterTabController*)tabController
				   withFilterListView:(FilterListView*)filterListView
							  atIndex:(NSUInteger)idx ;
- (void)dealloc;

- (BOOL)deleteFilter;

- (Prefs*)prefs;
- (ServerCommunication*)comm;
- (NSXMLElement*)filterE;
- (NSXMLElement*)filterDataE; // @Nullable for subs if server isn't running.
- (NSXMLElement*)ruleWithId:(int)ruleId;
- (void)checkReloadSubscriptionFilterData;

- (BOOL)isDirty;
- (void)notifyDirty;
- (void)notifyPublished;
- (void)notifyUpdatedAuthorizationState;

- (BOOL)enabled;
- (void)filterEnabled:(BOOL)enabled;

- (int)filterId;

- (NSString*)name;
- (void)setName:(NSString*)s;

- (NSString*)description;

- (NSString*)comments;
- (void)setComments:(NSString*)s;

- (BOOL)isSubscription;
- (BOOL)isPublished;
- (BOOL)isMobileMePublished;
- (BOOL)isWebDavPublished;
- (BOOL)safetySimpleRulesOnly;

- (NSString*)url;
- (void)setUrl:(NSString*)url;

- (BOOL)canEditName;
- (BOOL)canEditComments;
- (BOOL)canEditRules;
- (BOOL)canDelete;

- (void)startUpdateSubscription;

- (void)startEditingName;
- (void)makeTableFirstResponder;

- (NSNumber*)getRuleSubscriptionState:(NSString*)ruleId;
- (void)setRuleSubscriptionState:(NSNumber*)state forRuleId:(NSString*)ruleId;

- (void)readAttributesFromXml;

+ (NSString*)draggingDataType;

- (void)startEditingSubscription:(BOOL)oneshot;
- (IBAction)editCancelBtnAction:(id)sender;
- (IBAction)editOkBtnAction:(id)sender;

+ (NSString*)extractHostFromStringUrl:(NSString*)url;

@end

extern NSString* updatePolicies[];
extern NSString* safetyPolicies[];
