/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import "TableViewHelper.h"
#import "GBDebug.h"

#ifdef DEBUG
//#defined DEBUG_TABLEVIEW 1
#endif

@implementation TableViewHelper

- (BOOL)removeRow:(NSInteger)row fromTable:(NSTableView *)tableView
{
	NSInteger numRows = [tableView numberOfRows];
	DebugNSLog(@"removeRow %ld, num rows = %ld", row, numRows);
	if (row < 0) {
		NSLog(@"removeSelectedRows without selected row");
		return NO;
	}
	if (row >= numRows) {
		NSLog(@"Invalid row = %ld in removeSelectedRows, has only %ld rows", (long)row, (long)numRows);
		return NO;
	}
	if ([tableView editedRow] == row) {
		DebugNSLog(@"Ends editing as we're deleting the edited row");
		[[tableView window] endEditingFor:tableView];
	}
	if (![self deleteRowData:tableView forRow:row])
		return NO;
	[tableView reloadData];
	if (row + 1 < numRows) {
		DebugNSLog(@"Changes selection to next row");
		NSIndexSet* indexes = [NSIndexSet indexSetWithIndex:row]; // already deleted.
		[tableView selectRowIndexes:indexes byExtendingSelection:NO];
	} else if (row > 0) {
		DebugNSLog(@"Changes selection to previous row");
		NSIndexSet* indexes = [NSIndexSet indexSetWithIndex:row - 1];
		[tableView selectRowIndexes:indexes byExtendingSelection:NO];
	}
	return YES;
}

- (BOOL)removeSelectedRows:(NSTableView *)tableView
{
	NSIndexSet* set = [tableView selectedRowIndexes];
	for (NSInteger row = [set lastIndex]; row != NSNotFound; row = [set indexLessThanIndex:row]) {
		if (![self removeRow:row fromTable:tableView])
			return NO;
	}
	return YES;
}

- (BOOL)deleteRowData:(NSTableView *)tableView forRow:(NSUInteger)row
{
	NSAssert(NO, @"TableViewHelper.deleteRowData must be overridden.");
	return YES;
}

@end
