#!/bin/bash

set -e

if [ -s install-jars.sh ]
then
	cd ..
fi

if [ ! -f ./Proxy/install-jars.sh ]
then
	echo "Not in proper dir."
	exit 1
fi

if [ "$JARS_PATH" = "" ]
then
	JARS_PATH=/Library/PreferencePanes/GlimmerBlocker.prefPane/Contents/GlimmerBlockerProxy.app/Contents/Resources/Java
fi

#echo JARS_PATH: $JARS_PATH
if [ ! -d "$JARS_PATH" ]
then
	echo "Missing $JARS_PATH"
	exit 1
fi

GBTMP=`pwd`/../tmp
#echo GBTMP: $GBTMP

if [[ $(find ../tmp/classes -name GlimmerBlocker.class ) ]]
then
	true
else
	echo "Missing .class files.  Run make from within IntelliJ"
	exit 1
fi


if [[ $(find ../tmp/tcompile.dat -size +1024c ) ]]
then
	true
else
	echo "Missing templates. Run jannotemp.sh"
	exit 1
fi


mkdir -p $GBTMP/dyninfo 

perl -0ne 'die unless m{<key>CFBundleVersion</key>\s*<string>(.*?)</string>};print $1;' \
		 ./PreferencePane/src/PreferencePane-Info.plist > $GBTMP/dyninfo/gb-version.txt

TMPJAR=$GBTMP/gb-temp.jar
rm -rf $TMPJAR

add()
{
	WILDCARD=${2:-*}
	cd "$1" > /dev/null
	zip --no-dir-entries --quiet -0 --recurse-paths $TMPJAR -i \
		'*.class' '*.tmpldata' '*.js' '*.jpg' '*.gif' '*.png' '*.css' '*.txt' \
		-0 *
	cd - > /dev/null
}

for f in $GBTMP/classes/production/{G*,Jabhts-runtime,V*}
do
	add $f
done

add ./Proxy/src org/glimmerblocker/proxy
add $GBTMP/jannotemp-data

FINGERPRINT=$GBTMP/dyninfo/gb-fingerprint.txt
unzip -c $TMPJAR "*.class" | md5 > $FINGERPRINT

add $GBTMP/dyninfo

set -e
echo "Installs in $JARS_PATH"

DSTJAR=$JARS_PATH/glimmerblocker-server.jar

sudo rm -f "$DSTJAR"
sudo mv $TMPJAR "$DSTJAR"

for f in \
		./Rhino/jar-lib/smalljs.jar  \
		./Externals/intellij-annots/jar-lib/annotations.jar \
		./Externals/dnsjava/jar-lib/dnsjava.jar
do
	bn="`basename "$f"`"
	if ! diff --brief --new-file "$f" "$JARS_PATH/$bn" > /dev/null
	then
		echo "# Updates $f  -> $JARS_PATH/$bn"
		sudo rm -f "$JARS_PATH/$bn" # zap so current open files are not modified.
		sudo cp -f -p "$f" "$JARS_PATH/$bn"
	fi
done
