/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.globals.history;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class HistoryEntry
{
	/**
	 * The URL which was blocked/whitelisted/modified.
	 */
	@NotNull public final String url;

	/**
	 * Contains all actions: list of (ruleId, filterId, action+).
	 * The nested lists are encoded into a string using special unicode chars as separators.
	 *
	 * @see HistoryEntryBuilderImpl
	 */
	@NotNull public final String actions;

	/**
	 * The referer http header, if any.
	 */
	@Nullable public final String referer;

	/**
	 * Datestamp of execution.
	 */
	public final long time;

	HistoryEntry(@NotNull String url,
	             @Nullable String referer,
	             @NotNull String actions)
	{
		this.url = url;
		this.referer = referer;
		this.actions = actions;
		this.time = System.currentTimeMillis();
	}

	/**
	 * Partial:  referer and time is not part of the identity
	 */
	@Override
	public boolean equals(Object o)
	{
		if (this == o)
			return true;
		if (o == null || getClass() != o.getClass())
			return false;
		HistoryEntry that = (HistoryEntry)o;
		return url.equals(that.url) && actions.equals(that.actions);
	}

	/**
	 * Partial:  referer and time is not part of the identity
	 */
	@Override
	public int hashCode()
	{
		return actions.hashCode() * 31 + url.hashCode();
	}
}
