/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.globals.history;

import java.util.ArrayList;
import java.util.List;
import org.glimmerblocker.proxy.rule.AbstractRule;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

class HistoryEntryBuilderImpl implements HistoryEntryBuilder
{
	@NotNull private final HistoryCache cache;
	@NotNull private final String url;
	@Nullable private final String referer;
	@NotNull private final List<RuleEntry> ruleActionList;

	HistoryEntryBuilderImpl(@NotNull HistoryCache cache, @NotNull String url, @Nullable String referer)
	{
		this.cache = cache;
		this.url = url;
		this.referer = referer;
		this.ruleActionList = new ArrayList<>();
	}

	public void addAction(@NotNull AbstractRule rule, @NotNull String action)
	{
		// The list usually don't have more than 1 (maybe 2, 3) items, so linear search is ok.
		// Could be Cocoa-like and add HashMap when having many items.
		for (RuleEntry re : ruleActionList) {
			if (re.filterId == rule.filterId && re.ruleId == rule.ruleId) {
				re.actionsSB.append('\u2992'); // ⦒
				re.actionsSB.append(action);
				return;
			}
		}
		ruleActionList.add(new RuleEntry(rule.filterId, rule.ruleId, rule.isPreview, action));
	}

	public void commit()
	{
		if (ruleActionList.isEmpty())
			return;
		StringBuilder sb = new StringBuilder();
		for (RuleEntry re : ruleActionList) {
			if (sb.length() > 0)
				sb.append('\u2A1F'); // ⨟
			if (re.isPreview)
				sb.append('1');
			sb.append('\u29BF'); // ⦿
			sb.append(re.filterId);
			sb.append('\u29BF'); // ⦿
			sb.append(re.ruleId);
			sb.append('\u29BF'); // ⦿
			sb.append(re.actionsSB);
		}
		cache.addEntry(new HistoryEntry(url, referer, sb.toString()));
		ruleActionList.clear();
	}

	private static class RuleEntry
	{
		final int filterId;
		final int ruleId;
		final boolean isPreview;
		@NotNull final StringBuilder actionsSB;

		RuleEntry(int filterId, int ruleId, boolean isPreview, @NotNull String action)
		{
			this.filterId = filterId;
			this.ruleId = ruleId;
			this.isPreview = isPreview;
			this.actionsSB = new StringBuilder(action);
		}
	}
}
