/* Copyright (C) 2009-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.globals.network;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Immutable.  Fully keyable.
 */
public class HostnamePort implements Comparable<HostnamePort>
{
	@NotNull public final String hostname;
	@NotNull public final String hostnamePort;
	public final int port;


	public HostnamePort(@NotNull String hostname, int port)
	{
		this.hostname = hostname;
		this.port = port;
		this.hostnamePort = hostname + ":" + port;
	}

	/**
	 * @return null if param is null/empty.
	 */
	public static HostnamePort parse(@Nullable String s) throws Exception
	{
		if (s == null)
			return null;
		s = s.trim();
		if (s.length() == 0)
			return null;
		int idx = s.indexOf(':');
		int port = (idx < 0 || idx == s.length() - 1) ? 80 : Integer.parseInt(s.substring(idx + 1));
		if (port <= 0 || port > 65535)
			throw new Exception("Invalid port number = " + port);
		String hostname;
		if (idx < 0)
			hostname = s;
		else if (idx == 0)
			hostname = "127.0.0.1";
		else
			hostname = s.substring(0, idx);
		return new HostnamePort(hostname, port);
	}

	public static boolean equals(@Nullable HostnamePort a, @Nullable HostnamePort b)
	{
		return (a == null) ? b == null : a.equals(b);
	}

	@Override
	public boolean equals(Object o)
	{
		return (o instanceof HostnamePort)
				&& hostnamePort.equals(((HostnamePort)o).hostnamePort);
	}

	@Override
	public int hashCode()
	{
		return hostnamePort.hashCode();
	}

	public int compareTo(@NotNull HostnamePort k)
	{
		return hostnamePort.compareTo(k.hostnamePort);
	}

	@Override
	public String toString()
	{
		return "HostnamePort{" + hostnamePort + '}';
	}
}
