/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.http.client.connection;

import org.glimmerblocker.proxy.http.data.Headers;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class HttpResponseInfo
{
	public final long startTime;
	@NotNull public final Headers headers;
	@Nullable public final String transferEncoding;
	public final long keepAliveTimeoutSeconds;
	public final long contentLength;
	public final boolean isContentLessResponse;
	public final boolean canHaveNonEmptyBody;

	public HttpResponseInfo(long startTime,
	                        @NotNull String requestMethod,
	                        @NotNull Headers headers,
	                        @Nullable String transferEncoding,
	                        long keepAliveTimeoutSeconds,
	                        long contentLength)
	{
		this.startTime = startTime;
		this.headers = headers;
		this.transferEncoding = transferEncoding;
		this.keepAliveTimeoutSeconds = keepAliveTimeoutSeconds;
		this.contentLength = contentLength;
		int status = headers.status;
		this.isContentLessResponse = requestMethod.equals("HEAD")
				|| (status >= 100 && status <= 199) // All 1xx (informational)
				|| status == 204 // 204 = no content
				|| status == 304; // 304 = not modified
		this.canHaveNonEmptyBody = !isContentLessResponse && contentLength != 0;
	}
}
