/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.http.server;

import java.io.IOException;
import java.io.InputStream;
import org.jetbrains.annotations.NotNull;

class HttpServerContentLengthInputStream extends InputStream
{
	@NotNull final HttpServerRequest req;
	@NotNull final InputStream is;
	private long remaining;

	HttpServerContentLengthInputStream(@NotNull HttpServerRequest req, long remaining) throws IOException
	{
		this.req = req;
		this.is = req.conn.is;
		this.remaining = remaining;
		if (remaining < 0)
			throw new IOException("Invalid request ContentLength = " + remaining);
		didReadBytes(0);
	}

	public int read() throws IOException
	{
		if (remaining <= 0)
			return -1;
		int x = is.read();
		didReadBytes(1);
		return x;
	}

	@Override
	public int read(@NotNull byte[] data, int offset, int len) throws IOException
	{
		if (remaining <= 0)
			return -1;
		len = (int)Math.min(len, remaining);
		int num = is.read(data, offset, len);
		if (num > 0)
			didReadBytes(num);
		return num;
	}

	private void didReadBytes(int numBytes)
	{
		remaining -= numBytes;
		if (remaining == 0)
			req.hasReadFullRequestBody = true;
	}
}
