/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.rhino.callback;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.glimmerblocker.proxy.rule.ElementPlacementType;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import static java.util.regex.Pattern.CASE_INSENSITIVE;
import static java.util.regex.Pattern.DOTALL;
import static java.util.regex.Pattern.compile;

public class HtmlHelper
{
	public String html;

	public HtmlHelper(@Nullable String html)
	{
		this.html = html;
	}

	public void insertAtHeadStart(@Nullable String html, boolean forceInsert)
	{
		if (insert(html, false, headStartRE))
			return;
		insertAtHeadEnd(html, forceInsert);
	}

	public void insertAtHeadEnd(@Nullable String html, boolean forceInsert)
	{
		if (insert(html, true, headEndRE))
			return;
		if (insert("<!-- GB can't find <./head> -->\n" + html, true, bodyStartRE))
			return;
		if (insert("<!-- GB can't find <./head> or <.body> -->\n" + html, true, bodyEndRE))
			return;
		if (!forceInsert)
			return;
		if (html == null)
			html = "";
		this.html += "\n" +
				"<!-- GB can't find <./head>, <.body> or <./body> -->\n" +
				html;
	}

	public void insertAtBodyStart(@Nullable String s, boolean forceInsert)
	{
		if (insert(s, false, bodyStartRE))
			return;
		if (!forceInsert)
			return;
		if (html == null)
			html = "";
		html += "\n" +
				"<!-- GB can't find <.body> -->\n" +
				s;
	}

	public void insertAtBodyEnd(@Nullable String s, boolean forceInsert)
	{
		if (s == null || s.length() == 0)
			return;
		if (insert(s, true, bodyEndRE))
			return;
		if (!forceInsert)
			return;
		if (html == null)
			html = "";
		html += "\n" +
				"<!-- GB can't find <./body> -->\n" +
				s;
	}

	public void insert(@Nullable String s, @NotNull ElementPlacementType placement, boolean forceInsert)
	{
		switch (placement) {
			case HeadStart:
				insertAtHeadStart(s, forceInsert);
				break;
			case HeadEnd:
				insertAtHeadEnd(s, forceInsert);
				break;
			case BodyStart:
				insertAtBodyStart(s, forceInsert);
				break;
			case BodyEnd:
				insertAtBodyEnd(s, forceInsert);
				break;
			default:
				System.err.println("TransformContent.insert(" + placement + "): Missing case in switch.");
				insertAtBodyEnd(s, forceInsert);
		}
	}

	public boolean insert(@Nullable String s, boolean before, @NotNull Pattern re)
	{
		if (s == null || s.length() == 0 || html == null)
			return true;
		Matcher m = re.matcher(html);
		if (!m.find())
			return false;
		int idx = before ? m.start() : m.end();
		html = html.substring(0, idx) + s + html.substring(idx);
		return true;
	}

	public void replaceAll(@NotNull Pattern re, @NotNull String replacement)
	{
		if (html != null)
			html = re.matcher(html).replaceAll(replacement);
	}

	private static final Pattern headStartRE = compile("<head\\b.*?>[ \t]*\n?", CASE_INSENSITIVE | DOTALL);
	private static final Pattern headEndRE = compile("</head\\b", CASE_INSENSITIVE);
	private static final Pattern bodyStartRE = compile("<body\\b.*?>[ \t]*\n?", CASE_INSENSITIVE | DOTALL);
	private static final Pattern bodyEndRE = compile("</body\\b", CASE_INSENSITIVE);
}
