/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.util;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

/**
 * Utility for formatting dates.
 */
public class DateFormat
{
	/**
	 * @return Date in <a href='http://rfc.sunsite.dk/rfc/rfc1123.html'>RFC 1123</a> format used in HTTP headers.
	 */
	public static String to1123(long msecs)
	{
		return to1123(new Date(msecs));
	}

	/**
	 * @return Date in <a href='http://rfc.sunsite.dk/rfc/rfc1123.html'>RFC 1123</a> format used in HTTP headers.
	 */
	public static String to1123(Date d)
	{
		synchronized (df1123) { // SimpleDateFormat is not thread safe.
			return df1123.format(d);
		}
	}

	private static final SimpleDateFormat df1123 = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss z");

	static {
		df1123.setTimeZone(TimeZone.getDefault());
	}

}
