/* Copyright (C) 2008-2012 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import <Cocoa/Cocoa.h>


@interface AlertHelper : NSObject {
}

- (instancetype)initWindowView:(NSView*)windowView
				withMainBundle:(NSBundle*)mainBundle;

#pragma mark ------------------------ building blocks

- (void)prepareWarningAlertWithTitle:(NSString*)title
			  withInformativeText:(NSString*)infoText;

- (void)prepareCriticalAlertWithTitle:(NSString*)title
			   withInformativeText:(NSString*)infoText;

- (void)prepareInformativeAlertWithTitle:(NSString*)title
				  withInformativeText:(NSString*)infoText;


- (void)prepareAlertWithTitle:(NSString*)title
		  withInformativeText:(NSString*)infoText
					withStyle:(NSAlertStyle)alertStyle;
// NSInformationalAlertStyle
/*
 NSWarningAlertStyle = 0,
 NSInformationalAlertStyle = 1,
 NSCriticalAlertStyle = 2
 */


- (void)setDelegate:(NSObject*)callbackDelegate
	   withSelector:(SEL)callbackSelector;
// SEL    - (void)simpleCallbackWithReturnCode:(NSInteger)returnCode
// NSAlertFirstButtonReturn, NSAlertSecondButtonReturn, NSAlertThirdButtonReturn
@property (readonly) NSInteger lastAlertReturnCode;

- (NSButton*)addButtonWithTitle:(NSString*)title;

- (void)setAccessoryView:(NSView*)view;
- (void)layoutAlert;

- (void)showPreparedAlert;
- (void)dismissAlert; // does not invoke delegate
- (void)dismissAlertWithInvokeDelegate:(BOOL)callDelegate withReturnCode:(NSInteger)returnCode;

- (BOOL)canDisplayAlert; // false if no window or already displaying alert

- (NSWindow*)window;

@end
